from typing_extensions import Annotated

import uvicorn
from fastapi import FastAPI, status, HTTPException
from fastapi.responses import StreamingResponse
from pydantic import BaseModel, ConfigDict, Field, BeforeValidator, ValidationError

from .utils import convert_to_bytes, generate_data, load_database, save_database


class MaxSizePerRequestError(Exception):
    pass


class MinSizePerRequestError(Exception):
    pass


class Testdata:

    class Config(BaseModel):

        model_config = ConfigDict(extra='forbid')

        @staticmethod
        def to_bytes(value: int | str) -> int:
            try:
                return convert_to_bytes(value)
            except Exception as err:
                raise ValidationError from err

        @staticmethod
        def is_authorized_keys(value: set[str] | str) -> set[str]:
            if isinstance(value, str):
                with open(value, encoding='utf-8') as file:
                    return set(filter(lambda x: x.strip() != '', file.read().splitlines()))
            return value

        authorized_keys: Annotated[set[str], BeforeValidator(
            is_authorized_keys)] = Field(alias='keys')
        max_size: Annotated[int, BeforeValidator(
            to_bytes)] = Field(alias='max-size')
        max_data: Annotated[int, BeforeValidator(
            to_bytes)] = Field(alias='max-data')
        buffer_size: Annotated[int, BeforeValidator(
            to_bytes)] = Field(alias='buffer-size')
        database: str

    api: FastAPI
    config: Config

    def __init__(self, config: Config):
        self.config = config
        self.api = FastAPI(docs_url=None, redoc_url=None)

        @self.api.get('/zeros')
        async def zeros(api_key: str, size: str) -> StreamingResponse:
            try:
                if api_key not in config.authorized_keys:
                    raise HTTPException(
                        status_code=status.HTTP_401_UNAUTHORIZED,
                        detail='Invalid API Key.'
                    )

                try:
                    size = convert_to_bytes(size)
                except ValueError as err:
                    raise HTTPException(
                        status_code=status.HTTP_400_BAD_REQUEST,
                        detail='Invalid format format for size.'
                    ) from err

                if size < 0:
                    raise MinSizePerRequestError
                elif config.max_size < size:
                    raise MaxSizePerRequestError

                database = load_database(config.database)
                if config.max_data <= database['data-used'] + size:
                    raise HTTPException(
                        status_code=status.HTTP_500_INTERNAL_SERVER_ERROR,
                        detail='Service not available.'
                    )
                database['data-used'] += size

                save_database(config.database, database)

                return StreamingResponse(
                    status_code=status.HTTP_200_OK,
                    content=generate_data(size, config.buffer_size),
                    media_type='application/octet-stream',
                    headers={
                        'Content-Length': str(size)
                    }
                )

            except MinSizePerRequestError as err:
                raise HTTPException(
                    status_code=status.HTTP_416_REQUESTED_RANGE_NOT_SATISFIABLE,
                    detail='Size has to be not-negative.'
                ) from err
            except MaxSizePerRequestError as err:
                raise HTTPException(
                    status_code=status.HTTP_416_REQUESTED_RANGE_NOT_SATISFIABLE,
                    detail=f'Exceeded max size per request of {config.max_size} Bytes.'
                ) from err

    def run(self, host: str, port: int) -> None:
        uvicorn.run(self.api, host=host, port=port)
